#!/usr/bin/env python
#
# Problem EPage 414
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import numpy as np; from numpy import asarray, cos, sin, pi

from powell import powell

L = 1 # Meters 
P = 10 * 1.e3 # Newtons 
E = 200 * 1.e9

def objective_fn(x):
    """
    The function we want to minimize.
    """
    r_1, r_2 = x
    return L * pi * r_1**2 + L * pi * r_2**2 

def constraint_fns(x): 
    """
    A tuple of constraints we want to be <= 0.
    """
    r_1, r_2 = x
    sigma_1 = ( 8 * P * L ) / ( pi * r_1**3 ) 
    sigma_2 = ( 4 * P * L ) / ( pi * r_2**3 ) 
    delta = ( ( 4 * P * L**3 ) / ( 3 * pi * E ) ) * ( 7./r_1**4 + 1./r_2**4 ) 
    return ( -r_1, -r_2, sigma_1 - 180 * 1.e6, sigma_2 - 180 * 1.e6, delta - 25 * 1.e-3 )

def lagrangian_function(lam): 
    """
    Given a value of lambda returns a function of x that we can minimize with a minimization routine.
    """
    return lambda _ : F(_,lam)

def F(x,lam): 
    """
    Given lambda returns the value of the objective function 
    """
    objective = objective_fn(x)
    penalties = map( lambda _: max(_,0)**2, constraint_fns(x) )
    penalty = sum( penalties ) 
    return objective + lam * penalty 

# Start with an initial value of a Lagrangian value of lambda: 
lam = 1000.0
xStart = asarray( [pi,1.0] )
x, numIter = powell( lagrangian_function(lam), xStart, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point (mm)= ", x * 1000.
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)

# Increase lambda: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point (mm)= ", x * 1000.
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)




