#!/usr/bin/env python
#
# Problem EPage 416
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import numpy as np; from numpy import asarray, pi, sqrt, zeros

from powell import powell
from gaussElimin import gaussElimin

P = 200 * 1.e3 # Newtons 

def objective_fn(x):
    """
    The function we want to minimize.
    """
    a_1, a_2, a_3 = x 
    return ( a_1 + a_2 + a_3 ) # the "weight" of the total truss

def constraint_fns(x): 
    """
    A tuple of constraints we want to be <= 0.
    """
    a_1, a_2, a_3 = x 

    # Solve for the stresses in the beam:
    # 
    A = zeros( (3,3), dtype=float ) 
    b = zeros( (3,1), dtype=float ) 
    A[0,0] = a_1
    A[0,1] = ( 4./5 ) * a_2 
    A[1,1] = ( 3./5 ) * a_2 
    A[1,2] = a_3
    A[2,0] = 16./5
    A[2,1] = -5
    A[2,2] = 9./5
    b[0] = P 
    b[1] = P 
    gaussElimin(A,b) # the vector b is now the sigma_i 
    b -= ( 150 * 1.e3 ) # subtract the constant constraint from each sigma_i 
    return b 

def lagrangian_function(lam): 
    """
    Given a value of lambda returns a function of x that we can minimize with a minimization routine.
    """
    return lambda _ : F(_,lam)

def F(x,lam): 
    """
    Given lambda returns the value of the objective function 
    """
    objective = objective_fn(x)
    penalties = map( lambda _: max(_,0)**2, constraint_fns(x) )
    penalty = sum( penalties ) 
    return objective + lam * penalty 

# Start with an initial value of a Lagrangian value of lambda: 
lam = 1.
xStart = asarray( [1.,1.,1.] ) # A_1, A_2, A_3
x, numIter = powell( lagrangian_function(lam), xStart, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)

# Increase lambda: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)
