#!/usr/bin/env python
#
# Problem Page 308
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
from numpy import zeros,array,pi,sin
import pylab

from run_kut5 import integrate
from ridder import *
from printSoln import *

def initCond(u):  # Init. values of [y,y']; use 'u' if unknown
    return array([0.0, u])  

def r(u):         # Boundary condition residual--see Eq. (8.3)
    X,Y = integrate(F,xStart,initCond(u),xStop,h)
    y = Y[len(Y) - 1]
    r = y[0] # y(pi) = 0.
    return r

def F(x,y):       # First-order differential equations
    F = zeros(len(y))
    F[0] = y[1]
    F[1] = -sin(y[0]) - 1 
    return F

xStart = 0.0        # Start of integration
xStop = pi          # End of integration
u1 = 1/10           # 1st trial value of unknown init. cond.
u2 = 10.            # 2nd trial value of unknown init. cond.
h = 0.1             # Step size
freq = 2            # Printout frequency
u = ridder(r,u1,u2) # Compute the correct initial condition
X,Y = integrate(F,xStart,initCond(u),xStop,h)
printSoln(X,Y,freq)

if True: 
    pylab.plot( X, Y[:,0], color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.legend(loc='best')
    pylab.axis('tight')
    pylab.show() 
