#!/usr/bin/python
## example8_7
from numpy import zeros,array,arange
from newtonRaphson2 import *

#
# y'' = -2 y ( 2 x y' + y ) 
# y'(0) = 1/2 
# y(1) = -2/9 
#
# Solution
#
# y = ( 2 + x^2 )^{-1}
# 

def residual(y):  # Residuals of finite diff. Eqs. (8.11)
    r = zeros(m + 1)
    r[0] = y[0] - 1./2 
    y_mp1 = -(2./9)*(2*h) + y[m-1]
    r[m] = y[m-1] - 2*y[m] + y_mp1 + 2*h*x[m]*y[m]*y_mp1 - 2*h*x[m]*y[m]*y[m-1] + 2*y[m]*y[m]*h*h 
    for i in range(1,m):
        r[i] = y[i-1] - 2.0*y[i] + y[i+1] - h*h*F(x[i],y[i],(y[i+1] - y[i-1])/(2.0*h))
    return r                 
        
def F(x,y,yPrime):  # Differential eqn. y" = F(x,y,y')
    F = -2*y*( 2 * x * yPrime + y ) 
    return F

def startSoln(x): # Starting solution y(x)
    y = zeros(m + 1)
    for i in range(m + 1): y[i] = -(2./9)*x[i] + 1./2
    return y
 
xStart = 0.0            # x at left end
xStop = 1.0             # x at right end
m = 1000                # Number of mesh intevals
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
y = newtonRaphson2(residual,startSoln(x),1.0e-9)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i]) 

if True: 
    import pylab
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.hold('on')
    pylab.plot( x, 1./( 2 + x**2 ), color='green', label='true sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 


