#!/usr/bin/python
#
from numpy import zeros,ones,array,arange
from LUdecomp3 import *

#
# EPage 330 
#
# y''(x) = f(x) 
# y(0) = 0 
# y'(1/2) = 0
# 

gamma = 1.5

def rhs_fn(x):
    """
    """
    result = zeros( x.shape )

    inds_lt = ( 0 < x ) & ( x < 1./4 ) 
    result[inds_lt] = -(1./4) * x[inds_lt] 

    inds_gt = ( 1./4 < x ) & ( x < 1./2 ) 
    result[inds_gt] = -(1./(4*gamma)) * ( x[inds_gt] - 2 * ( x[inds_gt] - 1./4 )**2 )

    return result 

def equations(x,h,m): # Set up finite difference eqs.
    # y_{i-1} - 2 y_i + y_{i+1} = h^2 f(\xi_i) 
    h2 = h*h

    # Form the parts of A: 
    c = ones(m)
    d = -2 * ones(m+1)
    e = ones(m)
    d[0] = 1.0 # first equation is: y_0 = 0 
    e[0] = 0.0 
    c[m-1] = 2.0 # from the boundary conditions y'(1/2) = 0 we have 2 y_{m-1} - 2 y_m = h^2 f(\xi_m) \,.

    # Form the parts of b: 
    b = h2 * rhs_fn(x)
    b[0] = 0.0 

    return c,d,e,b

xStart = 0.0        # x at left end
xStop = 0.5         # x at right end
m = 1000            # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
c,d,e,b = equations(x,h,m)
c,d,e = LUdecomp3(c,d,e)
y = LUsolve3(c,d,e,b)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])
print "m= %5d; y'(1)= %6.3f" % ( m, (y[m]-y[m-1])/(x[m]-x[m-1]) )
print "max(y)= %10.6f, true max= %10.6f, diff= %10.6f" % ( max(y), 61./9216, 61./9216 - max(y) )

if True: 
    import pylab 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
