#!/usr/bin/python
#
from numpy import zeros,ones,array,arange,exp
from LUdecomp3 import *

delta = 1.5

def rhs_fn(x):
    """
    """
    numer = (1.-x) 
    denom = (1.+(delta-1)*x)**4
    return numer/denom 

def equations(x,h,m): # Set up finite difference eqs.
    """
    y_{i-1} + -2 y_i + y_{i+1} = -h^2 f(xi_i)
    """
    h2 = h*h

    # Form the parts of A = [c\d\e]:
    c = ones(m)
    d = -2 * ones(m+1)
    e = ones(m)
    d[0] = 1.0 # first equation is: y_0 = 0
    e[0] = 0.0 
    d[m] = 1.0 # last equation is: y_m = 0
    c[m-1] = 0.0 

    # Form the parts of b: 
    b = -h2*rhs_fn(x)
    b[0] = 0.0
    b[m] = 0.0
    return c,d,e,b

xStart = 0.0        # x at left end
xStop = 1.0         # x at right end
m = 100             # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
c,d,e,b = equations(x,h,m)
c,d,e = LUdecomp3(c,d,e)
y = LUsolve3(c,d,e,b)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])

numer = - (3 + 2*delta*x - 3*x) * x**2
denom = 6 * ( 1 + delta * x - x )**2
y_exact = numer / denom + x / (3*delta)

if True: 
    import pylab 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.hold('on')
    pylab.plot( x, y_exact, color='green', label='true sol.')
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
