#!/usr/bin/python
#
from numpy import zeros,array,arange,pi,sin
from newtonRaphson2 import *

#
# y'' = y^2 sin(y)
# y'(0) = 0 
# y(pi) = 1 
#
# 

def residual(y):  # Residuals of finite diff. Eqs. (8.11)
    r = zeros(m + 1)
    r[0] = -2 * y[0] + 2 * y[1] - y[0]**2 * sin(y[0]) * h*h
    r[m] = y[m] - 1.0
    for i in range(1,m):
        r[i] = y[i-1] - 2.0*y[i] + y[i+1] - h*h*F(x[i],y[i],(y[i+1] - y[i-1])/(2.0*h))
    return r                 
        
def F(x,y,yPrime):  # Differential eqn. y" = F(x,y,y')
    F = y**2 * sin(y)
    return F

def startSoln(x): # Starting solution y(x)
    y = zeros(m + 1)
    for i in range(m + 1): y[i] = 0.5*x[i]
    return y
 
xStart = 0.0            # x at left end
xStop = pi              # x at right end
m = 1000                 # Number of mesh intevals
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
y = newtonRaphson2(residual,startSoln(x),1.0e-5)
print "\n        x              y"
for i in [0,m]:
    print "%14.5e %14.5e" %(x[i],y[i]) 

if True: 
    import pylab 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
